#
# shellscript, generate xfig2.1-picture from daten.def
#  by Joerg.Schulenburg@physik.uni-magdeburg.de
#
# see /usr/doc/info/gawk.info.gz
#
# v1.4 Aug01 - 3d version
# v1.3 Feb01 - J1, J2 different (dashed)
# v1.2 Apr00 - circles around sites, better xfig-file
# v1.1 Sep99 - tail +n | head -1 replaced by awk
#
# TODO: use only bonds? (direction of bonds, easier to draw)
#
# the daten.def file should have additional lines for better drawing:
# #===base=== b1x b1y b2x b2y        - base vektors b1 b2 default=( 1.0 0.0 0.0 1.0 )
# #===edge=== e1x e1y e2x e2y        - set of edge-vectors used for PBC, units b1,b2
# #===pbcf=== x0 y0 x1 y1 .. [x0 y0] - PBC-frame for the cluster in units of b1,b2 (best choice: 0 0 e1x e1y e1x+e2x e1y+e2y  e2x e2y  0 0)
#
if test -z "$1"; then file=daten.def
                 else file=$1
fi
if test ! -r $file; then echo "file $file not found";exit; fi
#
# XFIG 2.1: 80dpi
#   line:   2 1 0 width=2 color=-1 0 0 0 0.000 -1 arrow=0 0\n\t x1 y1 x2 y2 9999 9999\n
#   polygon:2 3 0 width=1 color=2  0 0 0 0.000 -1 0 0\n\t %d %d ... 9999 9999\n
#   string: 4 justifyLCR=0 0 size=18 0 color=-1 0.00000 4 dy=17 dx=12 x y sting\001\n
#printf "#FIG 2.1\n80 2\n"
#
# XFIG 3.2: 1200dpi 
#   circle:1 3 0 width=1 pcolor=0  fcolor=7  depth=9 0 20 0.000 1 angle=0.0000 cx=6030 cy=1170 rx=229 ry=229 x1=6030 y1=1170 x2=6259 y2=1170\n
#   line:  2 1 style=0 width=2 pcolor=-1 fcolor=-1 depth=0 0 -1 dashlen=0.000 0 0 -1 0 0 points=2\n\t x1 y1 x2 y2\n
#              style:1=dashed
#   string:4 justifyLCR=0 color=-1 0 0 0 size=18 0.0000 4 dy=195 dx=n*135 x1 y1 string\001\n
printf "#FIG 3.2\nLandscape\nCenter\nMetric\nA4\n100.0\nSingle\n-2\n1200 2\n"
xx=$( awk '\
  BEGIN { count=0;nr=0;ox=0;oy=0;oz=0;rx=0;ry=0;rz=0 }
  !/^\#/ { count++; nr=count; } /^\#/ { nr=0; } (nr==1) { vn=$1;vnw=$2; }
  (nr > 1) && (nr < vn+2) {\
       if(($2)<ox) ox=($2); if(($2)>rx) rx=($2); 
       if(($3)<oy) oy=($3); if(($3)>ry) ry=($3);
       if(($4)<oz) oz=($4); if(($4)>rz) rz=($4);
     }
  END { printf " %f %f %f %f %f %f\n",ox,rx-ox,oy,ry-oy,oz,rz-oz }
  ' $file )
echo "# box=$xx (ox,rx,oy,ry,oz,rz)"
ox=$( echo "$xx" | awk '{ print -($1)+1.2 }' )
oy=$( echo "$xx" | awk '{ print $4+1 }' )
oz=$( echo "$xx" | awk '{ print $5 }' )
rz=$( echo "$xx" | awk '{ print $6 }' )
awk -v ox=$ox -v oy=$oy -voz=$oz -vrz=$rz '\
  function outs( x, y, z, s, c){ # number  c=color s=number
    printf "1 3 0 1 0 7 4 0 20 0.00 1 0.00 %d %d 230 230 %d %d %d %d\n",(x+ox)*sc,(y+oy)*sc,(x+ox)*sc,(y+oy)*sc,(x+ox)*sc+230,(y+oy)*sc;
    printf "4 1 %d 0 0 0 18 0.00000 4 195 135 %d %d %d\\001\n",c,(x+ox)*sc,(y+oy)*sc+90,s }
  function outc( x, y, z, s, c){ # string  c=color s=string
    printf "4 0 %d 0 0 0 18 0.00000 4 195 135 %d %d %s\\001\n",c,(x+ox)*sc+10,(y+oy)*sc+7,s }
  BEGIN { count=0;nr=0;b1x=1;b1y=0;b1z=0;b2x=0;b2y=-1;b2z=0;b3x=0;b3y=0;b3z=1;
          e1x=0;e1y=0;e2x=0;e2y=0;sc=80*15;d=3;
          for( i=0; i<d; i++ ) for( j=0; j<d; j++ ) { b[i*d+j]=0; e[i*d+j]=0 }
          for( i=0; i<d; i++ ) { b[i*d+i]=1; e[i*d+i]=1; }
          d=2; }
  !/^\#/ { count++; nr=count; }
  /^\#/  { nr=0 ; }
  /^\#===base===/  {
    if ( NF<10 )   { d=2 } else { d=3 }
    for( i=0; i<d; i++ ) for( j=0; j<d; j++ ) b[i*d+j]=$(2+i*d+j);
    b1x=$(2  );b1y=-($(3  ));
    b2x=$(2+d);b2y=-($(3+d));
    if(d>2){ b3x=$(2+2*d);b3y=-($(3+2*d));b3z=$(4+2d);b2z=$(4+d);b1z=$(4); };
  }
  /^\#===edge===/  {
    if( NF<10 ) { d=2 } else { d=3 }
    for( i=0; i<d; i++ ) for( j=0; j<d; j++ ) e[i*d+j]=$(2+i*d+j);
    e1x=(b1x*$(2    )+b2x*$(3    )); e1y=(b1y*$(2    )+b2y*$(3    )); 
    e2x=(b1x*$(2+  d)+b2x*$(3+  d)); e2y=(b1y*$(2+  d)+b2y*$(3+  d));
    if( d>2 ) {
      e3x=(b1x*$(2+2*d)+b2x*$(3+2*d)); e3y=(b1y*$(2+2*d)+b2y*$(3+2*d));
    }
  }
  /^\#===uvec===/  { # polyline2=1 thickness3=3 color4=4 forward_arrow10=1 depth=10
      s1="2 1 0 3 4 0 10 0 0.000 -1 1 0\n\t 0 0 3 16 22\n\t %d %d %d %d 9999 9999\n"
      s1="2 1 0 3 4 4 10 0 -1 0.000 0 0 -1 1 0 2\n\t 0 0 3 240 330\n\t %d %d %d %d\n"
      if ( NF < 10 ) { d=2 } else { d=3 }
      u1x=b1x*$(2  )+b2x*$(3  ); u1y=b1y*$(2  )+b2y*$(3  );
      u2x=b1x*$(2+d)+b2x*$(3+d); u2y=b1y*$(2+d)+b2y*$(3+d);
      printf s1,(ox-.5)*sc,(oy+.5)*sc,(u1x-.5+ox)*sc,(u1y+.5+oy)*sc;
      printf s1,(ox-.5)*sc,(oy+.5)*sc,(u2x-.5+ox)*sc,(u2y+.5+oy)*sc;
      outc( (u1x-.5),(u1y+.5),0,"b1",4);
      outc( (u2x-.5),(u2y+.5),0,"b2",4); }
  /^\#===pbcf===/  {
#     printf "2 3 0 1 2 0 0 0 0.000 -1 0 0\n\t";
      printf "2 3 0 1 2 2 3 0 -1 0.000 0 0 -1 0 0 %d\n\t",(NF-1)/d+1;
      for ( i=0; i+1<NF; i+=d ) {
         x=(b1x*$(i+2)+b2x*$(i+3)+ox)*sc;
         y=(b1y*$(i+2)+b2y*$(i+3)+oy)*sc; 
         printf " %d %d",x,y; } 
      x=(b1x*$(2)+b2x*$(3)+ox)*sc;
      y=(b1y*$(2)+b2y*$(3)+oy)*sc; 
      printf " %d %d",x,y;
#     printf " 9999 9999\n";
      printf "\n"; 
  }
  (nr == 1) { vn=$1;vnw=$2; printf "# NN=%d Nw=%d\n",vn,vnw }
  (nr > 1) && (nr < vn+2) {\
       ax[$1]=(b1x*$2+b2x*$3); 
       ay[$1]=(b1y*$2+b2y*$3);
       outs(ax[$1],ay[$1],0,$1,0);
  }
  (nr > vn+1) {\
       s1="2 1 0 %d -1 0 0 0 0.000 -1 0 0\n\t%d %d %d %d 9999 9999\n";
       s1="2 1 0 %d -1 0 5 0 -1 0.000 0 0 -1 0 0 2\n\t%d %d %d %d\n";
       if ($5>1) s1="2 1 1 %d -1 0 5 0 -1 6.000 0 0 -1 0 0 2\n\t%d %d %d %d\n";
       if ($5>2) s1="2 1 1 %d -1 0 5 0 -1 3.000 0 0 -1 0 0 2\n\t%d %d %d %d\n";
       if (match($0,"+"))
       { m1=0;m2=0;
         for( x1=-1; x1<2; x1++ ) for( x2=-1; x2<2; x2++ ) if( x1 || x2 ) # if( (x1!=0) || (x2!=0) )
         if ((ax[$2]+x1*e1x+x2*e2x-ax[$3])^2+(ay[$2]+x1*e1y+x2*e2y-ay[$3])^2\
            <(ax[$2]+m1*e1x+m2*e2x-ax[$3])^2+(ay[$2]+m1*e1y+m2*e2y-ay[$3])^2)
         { m1=x1;m2=x2; }
         printf s1,1,(ax[$2]+m1*e1x+m2*e2x+ox)*sc,(ay[$2]+m1*e1y+m2*e2y+oy)*sc,(ax[$3]+ox)*sc,(ay[$3]+oy)*sc;
         printf s1,1,(ax[$3]-m1*e1x-m2*e2x+ox)*sc,(ay[$3]-m1*e1y-m2*e2y+oy)*sc,(ax[$2]+ox)*sc,(ay[$2]+oy)*sc;
         outs( ax[$2]+m1*e1x+m2*e2x, ay[$2]+m1*e1y+m2*e2y, 0, $2, 1 );
         outs( ax[$3]-m1*e1x-m2*e2x, ay[$3]-m1*e1y-m2*e2y, 0, $3, 1 );
       }
       else { printf s1,2,(ax[$2]+ox)*sc,(ay[$2]+oy)*sc,(ax[$3]+ox)*sc,(ay[$3]+oy)*sc; }
  }
  END {
#    for( i=0; i<vn; i++ ) {
#       outs(ax[$1],ay[$1],0,$1,-1);
#    }
  }
  ' $file
exit

